#   shtub - shell command stub
#   Copyright (C) 2012 Immobilien Scout GmbH
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
    shtub - shell command stub.
"""

__author__ = 'Alexander Metzner, Michael Gruber, Udo Juettner'

import json

from os.path import join

from shtub.execution import Execution
from shtub.expectation import Expectation

VERSION = '0.2.9'

BASEDIR = 'shtub'

EXECUTIONS_FILENAME   = join(BASEDIR, 'executions')
EXPECTATIONS_FILENAME = join(BASEDIR, 'expectations')
LOCK_FILENAME         = join(BASEDIR, 'lock')
LOG_FILENAME          = join(BASEDIR, 'log')
STUBS_DIRECTORY       = join(BASEDIR, 'stubs')

READ_STDIN_TIMEOUT_IN_SECONDS = 1


def deserialize_expectations (filename):
    """
        loads the given json file and returns a list of expectations.
    """

    expectations = _load_json_file(filename)
    return list(map(lambda e: Expectation.from_dictionary(e), expectations))


def deserialize_executions (filename):
    """
        loads the given json file and returns a list of executions.
    """

    executions = _load_json_file(filename)
    return list(map(lambda e: Execution.from_dictionary(e), executions))


def serialize_executions (filename, executions):
    """
        writes the given execution objects into a json file with the given
        filename.
    """

    dictionaries = list(map(lambda e: e.as_dictionary(), executions))
    json_string = json.dumps(dictionaries, sort_keys=True, indent=4)

    with open(filename, mode='w') as json_file:
        json_file.write(json_string)


def _load_json_file (filename):
    """
        loads the given json file and returns the json content as dictionary.
    """

    with open(filename, mode='r') as json_file:
        file_content = json_file.read()
        dictionary = json.loads(file_content)

    return dictionary



#   shtub - shell command stub
#   Copyright (C) 2012 Immobilien Scout GmbH
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
    this module provides the class Verifier which offers methods to verify if
    the command stub has been called in the expected way. 
"""

__author__ = 'Alexander Metzner, Michael Gruber, Udo Juettner'

import os.path
from shtub import EXECUTIONS_FILENAME, deserialize_executions
from shtub.commandinput import CommandInput


class VerificationException (Exception):
    """
        to be raised when an error during verification occurs.
    """
    

class Verifier (object):
    """
        Verifies command stub expectations. Please use instances of this class
        in "with" statements.
    """
    
    def __init__ (self, basedir):
        """
            initializes a new verifier using the given base directory.
        """

        self.base_dir   = basedir
        self.executions = []


    def called (self, command):
        """
            raises an exception when no more executions are available or
            when the current execution does not have the expected command
            attribute, otherwise it will return the execution and remove the
            current execution from the list of executions.
        """
    
        if not self.executions:
            raise VerificationException('No more further executions: command "%s" can not be verified.' % command)
        
        actual_execution = self.executions[0]
        if actual_execution.command_input.command != command:
            raise VerificationException('Execution does not fulfill expectation:\n'
                                 'Expected command "%s", but got "%s"\n'
                                 % (command, actual_execution.command_input.command))
        
        self.executions = self.executions[1:]
        return VerfiableExecutionWrapper(actual_execution)
    
    
    def verify (self, command, arguments, stdin=None):
        """
            raises an exception when no more executions are available or
            when the current execution does not fulfill the given
            exception, otherwise it will pass and remove the current execution
            from the list of executions.
        """
    
        expected_input = CommandInput(command, arguments, stdin)

        if not self.executions:
            raise VerificationException('No more further executions, when verifying %s' % expected_input)
        
        actual_execution = self.executions[0]
        if not actual_execution.command_input.fulfills(expected_input):
            raise VerificationException('Execution does not fulfill expected_input:\n'
                                 'Expected %s\n'
                                 'Actual   %s\n'
                                 % (expected_input, actual_execution))
        
        self.executions = self.executions[1:]
    
    
    def __enter__ (self):
        """
            since this class is designed to be integrated in a "with" block it
            will load the actual executions and return itself.
        """
    
        filename = os.path.join(self.base_dir, EXECUTIONS_FILENAME)
        
        if not os.path.exists(filename):
            raise VerificationException('No executions found. Stubbed commands have never been called.')
    
        self.executions = deserialize_executions(filename)
        
        for execution in self.executions:
            if not execution.expected:
                raise VerificationException('Unexpected %s: did not fulfill any expectation.' % str(execution))
        
        return self


    def __exit__(self, exception_type, exception_value, traceback):
        """
            since this class is designed to be integrated in a "with" block it
            is implemented, but has no effect.

            @return: False, when exception_type, exception_value or traceback given,
                     otherwise None
        """
        
        if exception_type or exception_value or traceback:
            return False
        
        count_of_executions = len(self.executions)
        if count_of_executions > 0:
            if count_of_executions == 1:
                message = 'There is an unverified execution:\n'
            else:
                message = 'There are %s unverified executions:\n' % count_of_executions
                
            for execution in self.executions:
                message += "    %s\n" % str(execution)
                
            raise VerificationException(message)


class VerfiableExecutionWrapper (object):
    """
        Verifier.called returns this wrapper to make a fluent interface
        possible.
    """
    
    
    def __init__ (self, execution):
        """
            stores the given execution and assures and_input = with_input.
        """
        
        self.execution = execution
        self.and_input = self.with_input


    def at_least_with_arguments (self, *expected_arguments):
        """
            raises a exception if the expeceted arguments are not in the
            arguments of the wrapped execution. Returns the wrapper itself
            to make invocation chaining possible.
        """
        
        arguments = list(expected_arguments)
        
        for argument in arguments:
            if argument not in self.execution.command_input.arguments:
                raise VerificationException(
                    'Stub "%s" has not been executed with at least expected arguments %s, but with %s.'
                    % (self.execution.command_input.command, arguments, self.execution.command_input.arguments))
        
        return self


    def with_arguments (self, *expected_arguments):
        """
            raises a exception if the arguments of the wrapped execution are
            different than the expected arguments. Returns the wrapper itself
            to make invocation chaining possible.
        """
        
        arguments = list(expected_arguments)
        
        if self.execution.command_input.arguments != arguments:
            raise VerificationException(
                'Stub "%s" has not been executed with expected arguments %s, but with %s.'
                % (self.execution.command_input.command, arguments, self.execution.command_input.arguments))
        
        return self


    def with_input (self, expected_stdin):
        """
            raises a exception if the input from stdin in the wrapped execution
            is different than the expected stdin input. Returns the wrapper
            itself to make invocation chaining possible.
        """
        
        if self.execution.command_input.stdin != expected_stdin:
            raise VerificationException(
                'Stub "%s" has not received the expected stdin "%s", but got "%s".'
                % (self.execution.command_input.command, expected_stdin, self.execution.command_input.stdin))
        
        return self



#   shtub - shell command stub
#   Copyright (C) 2012 Immobilien Scout GmbH
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
    this module provides the class CommandINput, which represents tuple of
    all inputs a command stub receives: name of the command, arguments and
    the input from stdin.
"""

__author__ = 'Alexander Metzner, Michael Gruber, Udo Juettner'


class CommandInput (object):
    """
        Represents the parameters of a call to the command stub.
    """
    
    def __init__ (self, command, arguments, stdin):
        """
            initializes a new execution with the given properties.
            If arguments is not given it will be initialized as empty list.
        """
        
        self.command   = command
        self.arguments = arguments or []
        self.stdin     = stdin
    
    
    def as_dictionary (self):
        """
            returns a dictionary representation of the execution.
        """
        
        return {'command'   : self.command,
                'arguments' : self.arguments,
                'stdin'     : self.stdin}
    
    
    def fulfills (self, other):
        """
            tests if the command input fulfills the given other by comparing
            the command, the input from stdin and if at least the arguments
            of the other are given.
        """
        
        if self.command != other.command:
            return False
        
        if other.stdin != self.stdin:
            return False
        
        for argument in other.arguments:
            if argument not in self.arguments:
                return False
        
        return True


    def __eq__ (self, other):
        """
            returns True if the given execution has exactly the same properties.
        """
        
        return   self.command == other.command \
           and     self.stdin == other.stdin \
           and self.arguments == other.arguments

    
    def __ne__ (self, other):
        """
            returns True when __eq__ returns False or False when __eq__ returns
            True. 
        """
        
        return not(self == other)

    
    def __str__ (self):
        """
            returns a string representation using as_dictionary.
        """
        
        return 'CommandInput %s' % (self.as_dictionary())

    
    @staticmethod
    def from_dictionary (dictionary):
        """
            returns a new execution object with the properties from the given
            dictionary.
        """
        
        return CommandInput(dictionary['command'],
                            dictionary['arguments'],
                            dictionary['stdin'])



#!/usr/bin/env python
#
#   shtub - shell command stub
#   Copyright (C) 2012 Immobilien Scout GmbH
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
    the command stub.
"""

__author__ = 'Alexander Metzner, Michael Gruber, Udo Juettner'

import logging
import os
import sys
import fcntl

from select import select

from shtub import (BASEDIR,
                   EXECUTIONS_FILENAME,
                   EXPECTATIONS_FILENAME,
                   LOCK_FILENAME,
                   LOG_FILENAME,
                   READ_STDIN_TIMEOUT_IN_SECONDS,
                   deserialize_executions,
                   deserialize_expectations,
                   serialize_executions)

from shtub.execution import Execution
from shtub.commandinput import CommandInput


def lock ():
    """
        creates a file lock and blocks if the file lock is already locked.
    """

    logging.info('Acquire lock.')
    file_handle = open(LOCK_FILENAME, mode='a')
    fcntl.flock(file_handle, fcntl.LOCK_EX)

    logging.info('Lock acquired.')
    return file_handle


def unlock (file_handle):
    """
        releases the given file lock by closing it.
    """

    logging.info('Release lock.')
    file_handle.close()


def record_execution (execution):
    """
        loads the list of recent executions from the EXECUTIONS_FILENAME file,
        appends the given execution to the list, then writes the list back to
        the file again. To assure only one process is reading and writing the
        file a file lock is used.
    """

    lock_file_handle = lock()
    executions = []

    if os.path.exists(EXECUTIONS_FILENAME):
        executions = deserialize_executions(EXECUTIONS_FILENAME)

    executions.append(execution)
    serialize_executions(EXECUTIONS_FILENAME, executions)
    logging.info('Recorded %s executions.', len(executions))

    unlock(lock_file_handle)


def send_answer (answer):
    """
        writes the stdout and stderr as given in the answer and performs a
        sys.exit with the given return_code.
    """

    logging.info('Sending %s', answer)

    if answer.stdout is not None:
        sys.stdout.write(answer.stdout)

    if answer.stderr is not None:
        sys.stderr.write(answer.stderr)

    sys.exit(answer.return_code)


def dispatch (command_input):
    """
        currently this will handle the given command_input by testing whether it
        fulfills an expectation. If so it will save a execution (with the expected flag
        set to true) and send the next answer as defined in the expectation object.
    """

    expectations = deserialize_expectations(EXPECTATIONS_FILENAME)

    logging.info('Got %s', command_input)

    execution = Execution(command_input.command, command_input.arguments, command_input.stdin)
    
    for expectation in expectations:
        if command_input.fulfills(expectation.command_input):
            logging.info('Execution fulfills %s', expectation)
            
            execution.mark_as_expected()
            record_execution(execution)
            answer = expectation.next_answer()
            send_answer(answer)
            return

    logging.error('Given command_input does not fulfill requirements of any expectation.')
    sys.exit(255)


def read_stdin ():
    """
        waits READ_STDIN_TIMEOUT_IN_SECONDS seconds for input on stdin and
        is going to return the complete input if there is any. If there is no
        input it is going to return None.
    """

    read_list, _, _ = select([sys.stdin], [], [], READ_STDIN_TIMEOUT_IN_SECONDS)

    if len(read_list) > 0:
        return read_list[0].read()

    # quickfix: returning empty string instead of None, to make sure there are no differences
    #           between execution in tty and without.
    return ''


def handle_execution ():
    """
        creates the base directory, initializes the logging and will read in
        the arguments and input from stdin to create a new execution object.
    """

    if not os.path.exists(BASEDIR):
        os.mkdir(BASEDIR)

    logging_format = '%(asctime)s %(levelname)5s [%(name)s] process[%(process)d] thread[%(thread)d] - %(message)s'
    logging.basicConfig(filename=LOG_FILENAME,
                        level=logging.INFO,
                        format=logging_format)

    command   = os.path.basename(sys.argv[0])
    arguments = sys.argv[1:]
    stdin     = read_stdin()
    
    command_input = CommandInput(command, arguments, stdin)

    dispatch(command_input)


if __name__ == '__main__':
    handle_execution()



#   shtub - shell command stub
#   Copyright (C) 2012 Immobilien Scout GmbH
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
    This module provides a class called Answer, which represents the answer
    the command stub will send when an expectation is fulfilled.
"""

__author__ = 'Michael Gruber'

class Answer (object):
    """
        represents a answer the command stub is sending back when an expectation
        is fulfilled.
    """
    
    def __init__(self, stdout, stderr, return_code):
        """
            initializes with the given properties.
        """
        
        self.stdout      = stdout
        self.stderr      = stderr
        self.return_code = return_code
    
    
    def as_dictionary (self):
        """
            returns a dictionary representation of this object.
        """
        
        return {'stdout'      : self.stdout,
                'stderr'      : self.stderr,
                'return_code' : self.return_code}


    def __str__ (self):
        """
            returns a string representation of this object using the method
            as_dictionary.
        """
        
        return 'Answer %s' % (self.as_dictionary())


    def __eq__ (self, other):
        """
            returns True when the given answer object has exactly the same
            properties.
        """
        
        return      self.stdout == other.stdout \
           and      self.stderr == other.stderr \
           and self.return_code == other.return_code


    @staticmethod
    def from_dictionary (dictionary):
        """
            returns a new Answer object with the properties from the dictionary.
        """
        
        return Answer(dictionary['stdout'],
                      dictionary['stderr'],
                      dictionary['return_code'])



#   shtub - shell command stub
#   Copyright (C) 2012 Immobilien Scout GmbH
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

__author__ = 'Alexander Metzner, Michael Gruber, Udo Juettner'

import unittest
import os
import subprocess
import tempfile

from shtub import BASEDIR, STUBS_DIRECTORY
from shtub.fixture import Fixture
from shtub.verifier import Verifier

STUB_SCRIPT_CONTENT = """#!/usr/bin/env python
import shtub.commandstub

shtub.commandstub.handle_execution()
"""

class IntegrationTestBase (unittest.TestCase):
    def setUp (self):
        self.command_counter = 0
        self.set_base_dir(None)


    def _normalize_command_line(self, command):
        normalized = command.replace(' ', '_') \
                            .replace('*', '_') \
                            .replace('?', '_') \
                            .replace(':', '_') \
                            .replace('/', '_')
        return normalized


    def _write_output_file (self, command, stdout, stderr):
        normalized  = self._normalize_command_line(command)
        filename    = '%02d-%s' % (self.command_counter, normalized)
        output_path = os.path.join(self.base_dir, BASEDIR, filename)

        with open(output_path, 'wt') as output_file:
            output_file.write('--------------- ENVIRONMENT ----------------\n')
            for key in sorted(self.env.keys()):
                output_file.write('%s=%s\n' % (key, self.env[key]))

            output_file.write('----------------- STDOUT -------------------\n')
            output_file.write(str(stdout.decode("utf-8")))

            output_file.write('----------------- STDERR -------------------\n')
            output_file.write(str(stderr.decode("utf-8")))


    def execute_command (self, command):
        shell_process = subprocess.Popen(args=[command],
                                         stdout=subprocess.PIPE,
                                         stderr=subprocess.PIPE,
                                         shell=True,
                                         cwd=self.base_dir,
                                         env=self.env)

        stdout, stderr = shell_process.communicate()
        self._write_output_file(command, stdout, stderr)
        
        self.command_counter += 1

        return shell_process.returncode


    def fixture (self):
        return Fixture(self.base_dir)


    def verify (self):
        return Verifier(self.base_dir)


    def prepare_testbed (self, env, stubs):
        self.env   = env
        self.stubs = stubs

        os.mkdir(os.path.join(self.base_dir, BASEDIR))
        os.mkdir(self.stubs_dir)
        self.stub_commands(self.stubs)


    def stub_commands (self, command_list):
        for command in command_list:
            command_file_name = os.path.join(self.stubs_dir, command)
            
            with open(command_file_name, "w") as command_file:
                command_file.write(STUB_SCRIPT_CONTENT)
                
            os.chmod(command_file_name, 0o755)

    def make_base_dir (self, base_dir):
        os.makedirs(base_dir)
        self.set_base_dir(base_dir)


    def set_base_dir (self, base_dir):
        if base_dir:
            self.base_dir = base_dir
            self.cleanup_base_dir = False
        else:
            self.base_dir = tempfile.mkdtemp(prefix='integration-test-')
            self.cleanup_base_dir = True

        self.stubs_dir = os.path.join(self.base_dir, STUBS_DIRECTORY)



#   shtub - shell command stub
#   Copyright (C) 2012 Immobilien Scout GmbH
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
    this module provides the class Execution, which represents a call to the
    command stub.
"""

__author__ = 'Alexander Metzner, Michael Gruber, Udo Juettner'

from shtub.commandinput import CommandInput


class Execution (object):
    """
        Represents the parameters of a call to the command stub.
    """
    
    def __init__ (self, command, arguments, stdin, expected=False):
        """
            initializes a new execution with the given properties.
            If arguments is not given it will be initialized as empty list.
        """
        
        self.command_input = CommandInput(command, arguments, stdin)        
        self.expected      = expected
    
    
    def as_dictionary (self):
        """
            returns a dictionary representation of the execution.
        """
        
        return {'command_input' : self.command_input.as_dictionary(),
                'expected'      : self.expected}
    
    
    def mark_as_expected (self):
        """
            marks the execution as expected which means the execution fulfills a expectation.
        """
        
        self.expected = True


    def __eq__ (self, other):
        return self.command_input == other.command_input \
           and      self.expected == other.expected
 

    def __ne__ (self, other):
        return not(self == other)


    def __str__ (self):
        """
            returns a string representation using as_dictionary.
        """
        
        return 'Execution %s' % (self.as_dictionary())

    
    @staticmethod
    def from_dictionary (dictionary):
        """
            returns a new execution object with the properties from the given
            dictionary.
        """
        
        command_input_dictionary = dictionary['command_input']
        
        return Execution(command_input_dictionary['command'],
                         command_input_dictionary['arguments'],
                         command_input_dictionary['stdin'],
                         dictionary['expected'])



#   shtub - shell command stub
#   Copyright (C) 2012 Immobilien Scout GmbH
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
    this module provides the class Expectation which extends Execution with
    a list of answers and the index of the current answer.
"""

__author__ = 'Alexander Metzner, Michael Gruber, Udo Juettner'

from shtub.answer import Answer
from shtub.commandinput import CommandInput

class Expectation (object):
    """
        Represents the parameters of a expected command stub execution and
        contains corresponding answers.
    """
    
    # quickfix: stdin default is empty string to ensure no difference between execution
    #           in tty and without.
    def __init__ (self, command, arguments=[], stdin='', answers=[], initial_answer=0):
        """
            will initialize a new object with the given properties.
            answers and initial_answer are not mandatory.
        """
        
        self.command_input = CommandInput(command, arguments, stdin)
        
        self.answers        = []
        self.current_answer = initial_answer
        
        for answer in answers:
            self.answers.append(answer)
            
        self.and_input = self.with_input
    
    
    def as_dictionary (self):
        """
            returns a dictionary representation of this expectation.
        """
        
        answers_list = []
        
        for answer in self.answers:
            answer_dictionary = answer.as_dictionary()
            answers_list.append(answer_dictionary)
        
        result = {'command_input'  : self.command_input.as_dictionary(),
                  'answers'        : answers_list,
                  'current_answer' : self.current_answer}
                
        return result


    def next_answer (self):
        """
            returns the next answer in the list of answers or if the end of the
            list is reached it will repeatedly return the last answer of the
            list.
        """
        
        if len(self.answers) == 0:
            raise Exception('List of answers is empty!')
        
        result = self.answers[self.current_answer]
        
        if self.current_answer < len(self.answers) - 1:
            self.current_answer += 1
        
        return result
        
        
    def then (self, answer):
        """
            will append the given answer to the list of answers and return
            the object itself for invocation chaining.
        """
        
        self.answers.append(answer)
        
        return self
    
    
    def then_answer (self, stdout=None, stderr=None, return_code=0):
        """
            a convinience method to "then" which will create a new answer
            object with the given properties. 
        """
        
        return self.then(Answer(stdout, stderr, return_code))
        
        
    def then_return (self, return_code):
        """
            a convinience method to "then" which will create a new answer
            object with the given return_code. 
        """
        
        return self.then_answer(return_code=return_code)


    def then_write (self, stdout=None, stderr=None):
        """
            a convinience method to "then" which will create a new answer
            object with the given stdout and stderr output. 
        """
        
        return self.then_answer(stdout=stdout, stderr=stderr)
    
    
    def with_arguments (self, *arguments):
        """
            sets the given arguments and returns the expectation itself for
            invocation chaining
        """
        
        self.command_input.arguments = list(arguments)
        
        return self

    
    def with_input (self, stdin):
        """
            sets the given arguments and returns the expectation itself for
            invocation chaining
        """
        
        self.command_input.stdin = stdin
        
        return self


    def __eq__ (self, other):
        return  self.command_input == other.command_input \
           and self.current_answer == other.current_answer \
           and        self.answers == other.answers


    def __str__ (self):
        """
            returns a string representation of this expectation using
            as_dictionary
        """
        
        return 'Expectation %s' % (self.as_dictionary())
    
    
    @staticmethod
    def from_dictionary (dictionary):
        """
            returns a new expectation object with the properties from the given
            dictionary.
        """
        
        answers = []
        
        for answer_dictionary in dictionary['answers']:
            answer = Answer.from_dictionary(answer_dictionary)
            answers.append(answer)
        
        command_input_dictionary = dictionary['command_input']
        expectation = Expectation(command_input_dictionary['command'],
                                  command_input_dictionary['arguments'],
                                  command_input_dictionary['stdin'],
                                  answers,
                                  dictionary['current_answer'])
        
        return expectation



#!/usr/bin/env python

from setuptools import setup

if __name__ == '__main__':
    setup(
          name = 'shtub',
          version = '0.2.9',
          description = 'shtub - shell command stub',
          long_description = '''''',
          author = "Alexander Metzner, Michael Gruber, Udo Juettner",
          author_email = "alexander.metzner@gmail.com, aelgru@gmail.com, udo.juettner@gmail.com",
          license = 'GNU GPL v3',
          url = 'https://github.com/yadt/shtub',
          scripts = [],
          packages = ['shtub'],
          classifiers = ['Development Status :: 3 - Alpha', 'Programming Language :: Python'],
          
          
          
          
          zip_safe=True
    )



#   shtub - shell command stub
#   Copyright (C) 2012 Immobilien Scout GmbH
#
#   This program is free software: you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation, either version 3 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program.  If not, see <http://www.gnu.org/licenses/>.

"""
    this module provides the class Fixture, which represents the test fixture,
    e.g. it offers methods to create expectations for the command stub. 
"""

__author__ = 'Alexander Metzner, Michael Gruber, Udo Juettner'

import os

from shtub import EXPECTATIONS_FILENAME, serialize_executions
from shtub.expectation import Expectation

class Fixture (object):
    """
        Represents the testing context which can be configured by defining
        expectations and corresponding answers. Please use instances of this
        class in a "with" statement.
    """
    
    def __init__ (self, basedir):
        """
            initializes a new fixture with the given base directory.
        """
        
        self.base_dir     = basedir
        self.expectations = []
    
    # quickfix: expecting empty string instead of None, to make sure there's no difference
    #           between execution within tty and without
    def expect (self, command, arguments, stdin=''):
        """
            creates a new expectation with the given properties and appends it
            to the expectations, then returns the expectation for invocation
            chaining.
        """
        
        expectation = Expectation(command, arguments, stdin)
        self.expectations.append(expectation)
        
        return expectation
    
    
    def calling (self, command):
        """
            creates a new expecation with the given command and appends it to
            the expectations, then returns the expecation for invocation
            chaining.
        """
    
        expectation = Expectation(command)
        self.expectations.append(expectation)
        
        return expectation

    
    def __enter__ (self):
        """
            since this class is designed to be used using the "with" statement
            this returns the fixture itself.
        """
        
        return self
    
    
    def __exit__ (self, exception_type, exception_value, traceback):
        """
            since this class is designed to be used using the "with" statement
            this will save the list of expectations in the base directory.
            
            @return: False, when exception_type, exception_value or traceback given,
                     otherwise None
        """
         
        if exception_type or exception_value or traceback:
            return False
               
        filename = os.path.join(self.base_dir, EXPECTATIONS_FILENAME)
        
        serialize_executions(filename, self.expectations)
